package utils;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import ij.IJ;

import java.io.File;
import java.util.ArrayList;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import utils.InputOutput;
import utils.MyLog;
import utils.TableUtils;
import utils.UtilAyv;

public class InputOutputTest {

	@Before
	public void setUp() throws Exception {
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public final void testReadDoubleMatrixFromFile() {
		String matPath = "./data/mat02.txt";
		double[][] mat1 = InputOutput.readDoubleMatrixFromFile(matPath);
		MyLog.logMatrix(mat1, matPath);
	}

	@Test
	public final void testCreateDir() {

		String path = "./test33/";
		if (InputOutput.checkDir(path)) {
			boolean success = InputOutput.deleteDir(new File(path));
			assertTrue(
					"fallita cancellazione directory esistente, prima della creazione",
					success);
		}
		boolean success1 = InputOutput.createDir(new File(path));

		
		assertTrue("fallita creazione directory", success1);
		// assertTrue(InputOutput.checkDir("./test3/testAyv1/"));
	}

	@Test
	public final void testDeleteDir() {
		// assure directory exist
		String path = "./test33/";
		if (!InputOutput.checkDir(path)) {
			boolean success = InputOutput.createDir(new File(path));
			assertTrue(
					"fallita creazione directory inesistente, prima della cancellazione",
					success);
		}
		boolean success1 = InputOutput.deleteDir(new File(path));
		assertTrue("fallita cancellazione directory", success1);
	}

	@Test
	public final void testReadFile1() {
		String CODE_FILE = "/codici1.txt";
		String[][] expected = InputOutput
				.readStringMatrixFromFile("./data/codici1Read.txt");
		String tableCode[][] = new InputOutput().readFile1(CODE_FILE, 4);
		// Log.logMatrix(tableCode, "tableCode");
		assertTrue(TableUtils.compareTable(expected, tableCode));
	}

	@Test
	public final void testReadFile3() {
		String CODE_FILE = "/codici3.txt";
		String[][] expected = InputOutput
				.readStringMatrixFromFile("./data/codici3Read.txt");
		ArrayList<ArrayList<String>> tabella1 = new InputOutput()
				.readFile3(CODE_FILE);

		String[][] tableCode = new InputOutput()
				.fromArrayListToStringTable(tabella1);
		assertTrue(TableUtils.compareTable(expected, tableCode));
	}

	@Test
	public final void testReadFile5() {
		String CODE_FILE = "./data/iw2ayv.txt";
		String[][] expected = InputOutput
				.readStringMatrixFromFile("./data/iw2ayvRead.txt");
		ArrayList<ArrayList<String>> tabella1 = new InputOutput()
				.readFile5(CODE_FILE);
		String[][] tableCode = new InputOutput()
				.fromArrayListToStringTable(tabella1);
		// Log.logMatrix(tableCode, "tableCode");
		// TableUtils.dumpTable(tableCode, "tableCode");
		assertTrue(TableUtils.compareTable(expected, tableCode));
	}

	@Test
	public final void testIsCommentTrue() {
		String riga = "// Questo  un commento";
		boolean result = InputOutput.isComment(riga);
		assertTrue("fallito riconoscimento commento", result);
	}

	@Test
	public final void testIsCommentFalse() {
		String riga = "Questo NON  un commento";
		boolean result = InputOutput.isComment(riga);
		assertFalse("fallito riconoscimento NON commento", result);
	}

	@Test
	public final void testStripSlashComment() {
		String riga = "Questo programma / una// grandissima stronzata";
		String result = InputOutput.stripSlashComment(riga);
		// System.out.printf("\nresult= " + result);
		assertTrue(result.equals("Questo programma / una"));
	}

	@Test
	public final void testStripSlashCommentInesistent() {
		String riga = "Questo programma / una/ grandissima stronzata";
		String result = InputOutput.stripSlashComment(riga);
		// System.out.printf("\nresult= " + result);
		assertTrue(result
				.equals("Questo programma / una/ grandissima stronzata"));
	}

	@Test
	public final void testStripComment() {
		String riga = "Questo programma  una /* grandissima */stronzata";
		String result = InputOutput.stripComment(riga);
		// System.out.printf("\nresult= " + result);
		assertTrue(result.equals("Questo programma  una stronzata"));

	}

	@Test
	public final void testStripCommentInexistent() {
		String riga = "Questo programma  una grandissima */stronzata";
		String result = InputOutput.stripComment(riga);
		// System.out.printf("\nresult= " + result);
		assertTrue(result
				.equals("Questo programma  una grandissima */stronzata"));
	}

	@Test
	public final void testStripAllComments() {
		String riga = "Questo /*programma*/  /*una*/ cagata /* bellissimo*/ molto //cagoso";
		String result = InputOutput.stripAllComments(riga);
		// System.out.printf("\nresult= " + result);
		assertTrue(result.equals("Questo    cagata  molto "));

	}

	@Test
	public final void testReadDoubleArrayFromFile() {
		String TEST_FILE = "./data/vet05.txt";
		double[] expected = { 1048.927215, 1043.721519, 1042.943038,
				1042.183544, 1042.867089 };
		double[] result = InputOutput.readDoubleArrayFromFile(TEST_FILE);
		// Log.logVector(result, "result");
		assertTrue(UtilAyv.compareVectors(result, expected, 1e-8,
				"errore comparazione"));
	}

	@Test
	public final void testReadFloatArrayFromFile() {
		String TEST_FILE = "./data/vet05.txt";
		float[] expected = { (float) 1048.9272, (float) 1043.7216,
				(float) 1042.943, (float) 1042.1836, (float) 1042.8671 };
		float[] result = InputOutput.readFloatArrayFromFile(TEST_FILE);
		// Log.logVector(result, "result");
		assertTrue(UtilAyv.compareVectors(result, expected, (float) 1e-8,
				"errore comparazione"));

	}

	@Test
	public final void testReadIntArrayFromFile() {
		String TEST_FILE = "./data/vet04.txt";
		int[] expected = { 1, 54, 5678, 123, 5, 45, 777, 234, 0, };
		int[] result = InputOutput.readIntArrayFromFile(TEST_FILE);
		// attenzione, le righe bianche vengono lette come zeri
		// Log.logVector(result, "result");
		assertTrue(UtilAyv.compareVectors(result, expected,
				"errore comparazione"));
	}

	@Test
	public final void testReadStringArrayFromFile() {
		String TEST_FILE = "./data/vet05.txt";
		String[] expected = { "1048.927215", "1043.721519", "1042.943038",
				"1042.183544", "1042.867089" };

		String[] result = InputOutput.readStringArrayFromFile(TEST_FILE);
		// Log.logVector(result, "result");
		assertTrue(UtilAyv.compareVectors(result, expected,
				"errore comparazione"));

	}
	
	@Test
	public final void testAbsoluteToRelative() {
		String inStr = "C:\\2_eclipse_2\\eclipse\\workspace_fmri2\\ControlliMensili\\bin\\test3\\BTMA_01testP6";
		String outStr = InputOutput.absoluteToRelative(inStr);
		IJ.log(outStr);

	}

}
